//----------------------------------------------------------------------------//
//                                                                            //
//                           MC7601 board examples                            //
//                                                                            //
//                                Demo program                                //
//                         calculate factorial of 10.                         //
//                           Host part of program.                            //
//                                                                            //
//                  2014 (c) RC Module Inc., Moscow, Russia                   //
//                                                                            //
//----------------------------------------------------------------------------//

#include <iostream>
#include <stdlib.h>

#include "mc7601_ll_host.h"

using namespace std;

const unsigned int CORE_NUMBER = 0;
const int IP_ADDRESS = 0x0201A8C0; // 192.168.1.2

const char *NM_PART_FILE_NAME = "nm_part.abs";

int main()
{
	WR_Board *board;
	WR_Access *access;
	WR_channel_conf conf;
	WR_Word NM_part_return_value;
	int ret = 0;

	WR_SetChannel(conf, ANMETHERNET, IP_ADDRESS, 6901);

	if (WR_GetBoardDesc(&conf, &board) != WR_OK)
	{
		cout << "ERROR: Can't open board!\n";
		exit(1);
	}

	if (WR_GetAccess(board, CORE_NUMBER, &access) != WR_OK)
	{
		cout << "ERROR: Can't access core " << CORE_NUMBER << "!\n";
		exit(1);
	}

	if (WR_LoadInitCode(access) != WR_OK)
	{
		cout << "ERROR: Can't load init code!\n";
		exit(1);
	}

	if (WR_LoadProgramFile(access, NM_PART_FILE_NAME, 0xFFFFFFFF) != WR_OK)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Can't load program!\n";
		exit(1);
	}

	if (WR_WaitEndProgram(access, &NM_part_return_value, 0xFFFFFFFF) != WR_OK)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Can't wait end program!\n";
		exit(1);
	}

	if (NM_part_return_value == 3628800)
		cout << "10! = " << NM_part_return_value << endl;
	else
	{
		cout << "ERROR: 10! <> " << NM_part_return_value << endl;
		ret = 1;
	}

	if (WR_CloseAccess(access) != WR_OK)
	{
		cout << "ERROR: Can't close core!\n";
		exit(1);
	}

	if (WR_CloseBoardDesc(board) != WR_OK)
	{
		cout << "ERROR: Can't close board!\n";
		exit(1);
	}

	if (!ret)
		cout << "Test Ok.\n";

	return ret;
}
