//----------------------------------------------------------------------------//
//                                                                            //
//                           MC7601 board examples                            //
//                                                                            //
//                                Demo program                                //
//                  that exchange arrays with NMC3 on board.                  //
//                           Host part of program.                            //
//                                                                            //
//                  2014 (c) RC Module Inc., Moscow, Russia                   //
//                                                                            //
//----------------------------------------------------------------------------//

#include <iostream>
#include <stdlib.h>

#include "mc7601_ll_host.h"

using namespace std;

const unsigned int CORE_NUMBER = 0;
const int IP_ADDRESS = 0x0201A8C0; // 192.168.1.2

const char *NM_PART_FILE_NAME = "nm_part.abs";

int main()
{
	WR_Board *board;
	WR_Access *access;
	WR_channel_conf conf;
	WR_Word NM_part_return_value;

	WR_SetChannel(conf, ANMETHERNET, IP_ADDRESS, 6901);

	if (WR_GetBoardDesc(&conf, &board) != WR_OK)
	{
		cout << "ERROR: Can't open board!\n";
		exit(1);
	}

	if (WR_GetAccess(board, CORE_NUMBER, &access) != WR_OK)
	{
		cout << "ERROR: Can't access core " << CORE_NUMBER << "!\n";
		exit(1);
	}

	if (WR_LoadInitCode(access) != WR_OK)
	{
		cout << "ERROR: Can't load init code!\n";
		exit(1);
	}

	if (WR_LoadProgramFile(access, NM_PART_FILE_NAME, 0xFFFFFFFF) != WR_OK)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Can't load program!\n";
		exit(1);
	}

	WR_Word Length = 0;
	WR_Word *Buffer = 0;
	WR_Word AddrLoad = 0;
	WR_Word AddrStore = 0;
	WR_Word syncValue = 0;

	// Receive input array address
	if (WR_SyncArray(access, 0xFFFFFFFF,
		0, NULL, NULL,
		&syncValue, &AddrLoad, &Length)
		!= WR_OK)
	{
		cout << "Error in sync 1.\n";
		return 2;
	}

	if (syncValue != 1)
	{
		cout << "Error value in sync 1.\n";
		return 2;
	}

	Buffer = new WR_Word[Length];

	for (int i = 1; i < 31; i++)
	{
		// Sync for receive array and address of output array
		if (WR_SyncArray(access, 0xFFFFFFFF,
			1, NULL, NULL,
			&syncValue, &AddrStore, NULL)
			!= WR_OK)
		{
			cout << "Error in sync 2.\n";
			return 2;
		}

		if (syncValue != 2)
		{
			cout << "Error value in sync 2.\n";
			return 2;
		}

		// Receive array
		if (WR_ReadMemBlock(access, Buffer, AddrLoad, Length) != WR_OK)
		{
			cout << "Error in read.\n";
			return 1;
		}

		cout << "Verify number " << i << ".\n";

		for (int j = 0; j < Length; j++)
		{
			if (Buffer[j] != i)
			{
				cout << "Bad buffer[" << j << "] on step " << i << ".\n";
				return 3;
			}

			// New fill of array
			Buffer[j] = i + 1;
		}

		if (WR_WriteMemBlock(access, Buffer, AddrStore, Length) != WR_OK)
		{
			cout << "Error in write.\n";
			return 1;
		}

		// Sync for send array
		if (WR_Sync(access, 0xFFFFFFFF,
			2, &syncValue)
			!= WR_OK)
		{
			cout << "Error in sync 3.\n";
			return 2;
		}

		if (syncValue != 3)
		{
			cout << "Error value in sync 3.\n";
			return 2;
		}
	}

	// Sync for end program
	if (WR_Sync(access, 0xFFFFFFFF,
		17, NULL)
		!= WR_OK)
	{
		cout << "Error value in sync 4.\n";
		return 2;
	}

	delete Buffer;

	if (WR_WaitEndProgram(access, &NM_part_return_value, 0xFFFFFFFF) != WR_OK)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Can't wait end program!\n";
		exit(1);
	}

	if (NM_part_return_value)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Wrong return value (" << NM_part_return_value << ")!\n";
		exit(1);
	}

	if (WR_CloseAccess(access) != WR_OK)
	{
		cout << "ERROR: Can't close core!\n";
		exit(1);
	}

	if (WR_CloseBoardDesc(board) != WR_OK)
	{
		cout << "ERROR: Can't close board!\n";
		exit(1);
	}

	cout << "Test Ok.\n";
	return 0;
}
