// Interface PC-ARM from РС (host)

#ifndef __INTERFACE_PC_ARM__H__
#define __INTERFACE_PC_ARM__H__

#include <windows.h>

#ifdef _MAKE_DLL
#define DECLSPEC __declspec(dllexport)
#else
#define DECLSPEC __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

// Version
#define VERSION_MAJOR 1
#define VERSION_MINOR 0

typedef unsigned int SP_Word;
typedef unsigned int SP_Addr;

// Types of PC-ARM channels
enum SP_channel {
	ANMUSB      = 0, // TEMPORARY NOT WORK - NOT USE!!!
	ANMETHERNET = 1,
	ANMSERIAL   = 2, // NOT USE!!!
	ANMSPI      = 3  // NOT USE!!!
};

// Definition of PC-ARM channel
typedef struct _SP_channel_conf {
	int channel; // (USE ANMETHERNET)
	int param1;  // ANMETHERNET - IP
	int param2;  // ANMETHERNET - port
} SP_channel_conf;

// Count of cores
#define SP_cores_count 2

#define IO_BUFSZ_SM 0xF000

// Shared memory
typedef struct _SH_MEM_STRUCT {
	int Board_cnt; // Количество открытых дескрипторов плат
	int Daemon_PID; // PID демона
	SP_channel_conf Daemon_channel_conf; // Конфигурация канала
	int Daemon_result; // Возвращаемое значение демона
	int Daemon_cmd;
	size_t Daemon_ln;
	int IO_request; // 1 - emulate event signal
	char BUFFER[IO_BUFSZ_SM];
} SH_MEM_STRUCT;

#define SP_tag_size 8

// Board descriptor
typedef struct _SP_Board {
	char tag[SP_tag_size]; // "SP_Board"
	int status;
	unsigned int cnt_reference[SP_cores_count]; // Count of opened cores descriptors
	HANDLE hMutex_Channel; // Mutex for channel
	HANDLE hSharedMapFile; // File-mapping object
	LPVOID lpSharedMapAddress; // Мапированный file-mapping object
	SH_MEM_STRUCT *SH_MEM; // Мапированный file-mapping object (копия)
	HANDLE hEvent_Channel; // Event
	HANDLE hTask_Mutex; // Mutex for task
	char *IO_Buffer;
	DWORD Channel_timeout; // Таймаут на освобождение канала
} SP_Board;

// Core descriptor
typedef struct _SP_Access {
	char tag[SP_tag_size]; // "SP_Acces"
	int NMno;
	int status;
	SP_Board *board;
} SP_Access;

// Result SP_* functions
enum SP_RetValue {
	SP_OK              =   0, // OK
	SP_FAIL            =   1, // Error
	SP_CANCEL          =   2, // Cancel
	SP_AGAIN           =   3, // Try again
	SP_TIMEOUT         =   4, // Timeout
	SP_FILE            =   5, // Can not find file
	SP_BADADDRESS      =   6, // Bad address ranges
	SP_PC_DMN_FAIL     =   7, // Error with PC daemon
	SP_CONNECTION      =   8, // Error during setting connetion with ARM
	SP_CHANNEL_TIMEOUT =   9, // Waiting free CHANNEL timeout
	SP_CHANNEL_FAIL    =  10, // Error with CHANNEL
	SP_ARM_DMN_FAIL    =  11, // Error with ARM daemon
	SP_ARM_DMN_AGAIN   =  12, // ARM daemon say: try again
	SP_NOT_SUPPORTED   = 127  // Not supported
};

// Result PL_* functions
enum PL_RetValue {
	PL_OK            =   0, // OK
	PL_ERROR         =   1, // Error
	PL_CANCEL        =   2, // Cancel
	PL_AGAIN         =   3, // Try again
	PL_TIMEOUT       =   4, // Timeout
	PL_FILE          =   5, // Can not find file
	PL_BADADDRESS    =   6, // Bad address ranges
	PL_DMN_ERROR     =   7, // ARM daemon error
	PL_DMN_AGAIN     =   8, // ARM daemon say: try again
	PL_NOT_SUPPORTED = 127  // Not supported
};

//----------------------------------------------------------------------------//
// Функции определяющие доступ к ЦУПП                                         //
//----------------------------------------------------------------------------//

// Получение дескриптора доступа к плате ЦУПП
DECLSPEC int __stdcall SP_GetBoardDesc(SP_channel_conf *channel_conf, SP_Board **board);

// Закрытие дескриптора доступа к плате ЦУПП
DECLSPEC int __stdcall SP_CloseBoardDesc(SP_Board *board); 

//----------------------------------------------------------------------------//
// Функции работающие с экземпляром ЦУПП                                      //
//----------------------------------------------------------------------------//

// Получение версии библиотеки
DECLSPEC int __stdcall SP_GetVersion(SP_Board *board, unsigned int *version_major, unsigned int *version_minor);

// Получение дескриптора доступа к ядру ЦУПП с номером coreNo
// coreNo может принимать значения: 0 – NM1; 1 – NM2.
DECLSPEC int __stdcall SP_GetAccess(SP_Board *board, unsigned int coreNo, SP_Access **access);

// Закрытие дескриптора доступа к ядру ЦУПП
DECLSPEC int __stdcall SP_CloseAccess(SP_Access *access);

//----------------------------------------------------------------------------//
// Функции работающие с процессором ЦУПП                                      //
//----------------------------------------------------------------------------//

// Квази сброс ядра
DECLSPEC int __stdcall SP_ResetCore(SP_Access *access);

// Загрузка кода инициализации на ядро
DECLSPEC int __stdcall SP_LoadInitCode(SP_Access *access);

// Загрузка пользовательской программы на ядро
DECLSPEC int __stdcall SP_LoadProgram(SP_Access *access, void *addrProgram, unsigned int sizeProgram, DWORD timeOut);

// Загрузка файла пользовательской программы на ядро
DECLSPEC int __stdcall SP_LoadProgramFile(SP_Access *access, const char *filename, DWORD timeOut);

// Ожидание завершения пользовательской программы и получение её возвращаемого значения
DECLSPEC int __stdcall SP_WaitEndProgram(SP_Access *access, SP_Word *returnValue, DWORD timeOut);

// Чтение массива из памяти NM
DECLSPEC int __stdcall SP_ReadMemBlock(
	SP_Access *access,
	SP_Word *block,
	SP_Addr address,
	unsigned int len);

// Запись массива в память NM
DECLSPEC int __stdcall SP_WriteMemBlock(
	SP_Access *access,
	SP_Word *block,
	SP_Addr address,
	unsigned int len);

// Барьерная синхронизация с программой на NM
DECLSPEC int __stdcall SP_Sync(SP_Access *access, DWORD timeOut, SP_Word value, SP_Word *returnValue);

// Барьерная синхронизация массивом с программой на NM
DECLSPEC int __stdcall SP_SyncArray(
	SP_Access *access,
	DWORD timeOut,

	SP_Word value,
	SP_Addr outAddress,
	SP_Word outLen,

	SP_Word *returnValue,
	SP_Addr *inAddress,
	SP_Word *inLen);

// Посылка NMI на ядро
DECLSPEC int __stdcall SP_InterruptNMI(SP_Access *access);

// Посылка HPINT на ядро
DECLSPEC int __stdcall SP_InterruptHPINT(SP_Access *access);

// Посылка LPINT на ядро
DECLSPEC int __stdcall SP_InterruptLPINT(SP_Access *access);

//----------------------------------------------------------------------------//
// Дополнительные функции библиотеки для многозадачного доступа               //
//----------------------------------------------------------------------------//

// Установка максимального времени ожидания освобождения канала
DECLSPEC int __stdcall SP_SetChannelTimeout(SP_Board *board, DWORD timeOut);

// Получение параметров канала
DECLSPEC int __stdcall SP_GetChannelInf(SP_Board *board, int *chan, int *p1, int *p2, DWORD *timeOut);

// Определение количества доступных экземпляров ЦУПП
DECLSPEC int __stdcall SP_GetBoardCount(int *count);

//----------------------------------------------------------------------------//
// Недокументированные функции                                                //
//----------------------------------------------------------------------------//

#ifdef UNDOC

// Чтение регистра (ячейки памяти)
DECLSPEC int __stdcall SP_GetRegister(SP_Access *access, SP_Addr addr, SP_Word *val);

// Запись в регистр (ячейку памяти)
DECLSPEC int __stdcall SP_SetRegister(SP_Access *access, SP_Addr addr, SP_Word val);

#endif // UNDOC

#ifdef __cplusplus
}
#endif

#endif // __INTERFACE_PC_ARM__H__
